from rptk.configuration import Config


class Dispatcher(object):
    def __init__(self, config=None):
        if not isinstance(config, Config):
            raise TypeError("%s not of type %s" % (config, Config))
        self._config = config
        self._query_class = config.args.query_class
        self._format_class = config.args.format_class
        self._object = config.args.object
        self._name = config.args.name

    @property
    def config(self):
        return self._config

    def dispatch(self):
        with self.config.args.query_class(config=self.config) as q:
            result = q.query(obj=self.config.args.object)
        with self.config.args.format_class(config=self.config) as f:
            output = f.format(result=result, name=self.config.args.name)
        return output



import logging
import jinja2
from rptk.configuration import Config


class BaseFormat(object):
    def __init__(self, config=None):
        if not isinstance(config, Config):
            raise TypeError("%s not of type %s" % (config, Config))
        self._config = config
        self._log = logging.getLogger(__name__)
        self._log.addHandler(config.logging_handler)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    @property
    def config(self):
        return self._config

    @property
    def log(self):
        return self._log

    def format(self, result=None, name=None):
        if not isinstance(result, dict):
            raise TypeError("%s not of type %s" % (result, dict))
        if not isinstance(name, basestring):
            raise TypeError("%s not of type %s" % (name, basestring))
        return None


class JinjaFormat(BaseFormat):
    template_name = None

    def __init__(self, config=None):
        super(JinjaFormat, self).__init__(config=config)
        self.env = jinja2.Environment(
            loader=jinja2.PackageLoader('rptk')
        )
        self.env.trim_blocks = True
        self.env.lstrip_blocks = True
        self._template = None

    @property
    def template(self):
        return self._template

    def load_template(self):
        self._template = self.env.get_template(self.template_name)

    def format(self, result=None, name=None):
        super(JinjaFormat, self).format(result=result, name=name)
        self.load_template()
        if isinstance(self.template, jinja2.Template):
            output = self.template.render(result=result, name=name)
        else:
            raise TypeError("%s not of type %s" % (self.template, jinja2.Template))
        return output



import os
import importlib
import ConfigParser
import argparse
import logging


class Config(object):
    def __init__(self, argv=None, opts=None):
        if not argv:
            argv = list()
        parser = argparse.ArgumentParser(add_help=False)
        config = ConfigParser.SafeConfigParser()
        default_config_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'rptk.conf')
        parser.add_argument(
            '--config_path', '-f', action='store', type=str,
            help="path to configuration file", default=default_config_path
        )
        partial_args, remaining_args = parser.parse_known_args(args=argv)
        config.read(partial_args.config_path)
        try:
            defaults = dict(config.items("defaults"))
        except ConfigParser.Error:
            raise
        if opts:
            if not isinstance(opts, dict):
                raise TypeError("%s not of type %s" % (opts, dict))
            defaults.update(opts)
        query_classes = self._register_classes(config.items("query-classes"))
        format_classes = self._register_classes(config.items("format-classes"))
        parser.set_defaults(**defaults)
        parser.add_argument(
            '--query', '-Q', action='store', type=str,
            help="query class", choices=query_classes.keys()
        )
        parser.add_argument(
            '--format', '-F', action='store', type=str,
            help="format class", choices=format_classes.keys()
        )
        parser.add_argument('--host', action='store', type=str, help="irrd host to connect to")
        parser.add_argument('--port', action='store', type=int, help="irrd service tcp port")
        parser.add_argument('--name', action='store', type=str, help="prefix-list name (default: object)")
        parser.add_argument('--help', action='help', help="print usage information and exit")
        parser.add_argument('object', nargs='?', action='store', type=str, help="rpsl object name")
        args = parser.parse_args(args=remaining_args)
        if not args.object:
            raise RuntimeError("No default value provided for object")
        if not args.name:
            args.name = args.object
        args.query_class = query_classes[args.query]
        args.format_class = format_classes[args.format]
        self._logging_handler = logging.StreamHandler()
        self._args = args

    @property
    def args(self):
        return self._args

    @property
    def logging_handler(self):
        return self._logging_handler

    def _register_classes(self, entries=None):
        classes = dict()
        for entry in entries:
            name = entry[0]
            mod_path, cls_path = entry[1].rsplit(".", 1)
            cls = getattr(importlib.import_module(mod_path), cls_path)
            classes.update({name: cls})
        return classes



import json
from rptk.modules.format import BaseFormat


class JsonFormat(BaseFormat):
    def format(self, result=None, name=None):
        super(JsonFormat, self).format(result=result, name=name)
        output = json.dumps({unicode(name): result}, indent=4)
        return output



import sys
from rptk import configuration, dispatch


def main():
    argv = sys.argv[1:]
    config = configuration.Config(argv=argv)
    dispatcher = dispatch.Dispatcher(config=config)
    result = dispatcher.dispatch()
    sys.stdout.write("%s\n" % result)
    return


if __name__ == "__main__":
    main()



from flask_api import FlaskAPI
from flask_api.renderers import BaseRenderer, BrowsableAPIRenderer
from flask_api.decorators import set_renderers
from rptk.api import Rptk

app = FlaskAPI(__name__)


class TextRenderer(BaseRenderer):
    media_type = 'text/plain'

    def render(self, data, media_type, **options):
        return "%s\n" % data


@app.route("/<string:fmt>/<string:obj>", methods=['GET'])
@set_renderers(TextRenderer)
def get(fmt=None, obj=None):
    rptk = Rptk(object=obj, format=fmt)
    result = rptk.query()
    return result


if __name__ == "__main__":
    app.run(host='0.0.0.0', port=8080)



import yaml
from rptk.modules.format import BaseFormat


class YamlFormat(BaseFormat):
    def format(self, result=None, name=None):
        super(YamlFormat, self).format(result=result, name=name)
        output = yaml.dump(
            {unicode(name): result},
            indent=4,
            explicit_start=True,
            explicit_end=True,
            default_flow_style=False,
        )
        return output



import json
from rptk.configuration import Config


class BaseFormatter(object):
    def __init__(self, config=None):
        if not isinstance(config, Config):
            raise TypeError("%s not of type %s" % (config, Config))
        self._config = config

    def format(self, result=None, name=None):
        if not isinstance(result, dict):
            raise TypeError("%s not of type %s" % (result, dict))
        if not isinstance(name, str):
            raise TypeError("%s not of type %s" % (name, str))
        return None


class TestFormatter(BaseFormatter):
    def format(self, result=None, name=None):
        super(TestFormatter, self).format(result=result, name=name)
        output = {
            name: result
        }
        return output


class JsonFormatter(BaseFormatter):
    def format(self, result=None, name=None):
        super(JsonFormatter, self).format(result=result, name=name)
        output = json.dumps({name: result}, indent=4)
        return output


class IosFormatter(BaseFormatter):
    def format(self, result=None, name=None):
        super(IosFormatter, self).format(result=result, name=name)
        output = str()
        for af in result:
            i = 0
            for entry in result[af]:
                i += 10
                output += self.line(name=name, af=af, entry=entry, i=i)
        return output

    def line(self, name='Prefix-List', af=None, entry=None, i=None):
        if not isinstance(entry, dict):
            raise ValueError("%s not of type %s" % (entry, dict))
        if not isinstance(i, int):
            raise ValueError("%s not of type %s" % (i, int))
        return "%s %s seq %s permit %s\n" % (self.af(af=af), name, i, entry['prefix'])

    def af(self, af=None):
        if af == 'ipv4':
            return 'ip prefix-list'
        elif af == 'ipv6':
            return 'ipv6 prefix-list'
        else:
            raise ValueError("%s is not a valid address-family name" % af)



import os
from unittest import TestCase
from whichcraft import which
from rptk import configuration, dispatch


class TestRptk(TestCase):
    config_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'tests.conf')
    argv = ['--config_path', config_path]

    def test_01_dependency_bgpq3(self):
        path = which('bgpq3')
        self.assertIsInstance(path, str, msg="bgpq3 executable not found in PATH")

    def test_02_configure(self):
        config = configuration.Config(argv=self.argv)
        self.assertIsInstance(config, configuration.Config, msg="config object invalid")
        self.config = config

    def test_03_dummy_query(self):
        config = configuration.Config(argv=self.argv)
        dispatcher = dispatch.Dispatcher(config=config)
        result = dispatcher.dispatch()
        self.assertIsInstance(result, dict)
        self.assertIsInstance(result["test"], dict)
        self.assertEqual(result["test"]["object"], config.args.object)



import logging
from rptk.configuration import Config


class BaseQuery(object):
    def __init__(self, config=None):
        if not isinstance(config, Config):
            raise TypeError("%s not of type %s" % (config, Config))
        self._config = config
        self._log = logging.getLogger(__name__)
        self._log.addHandler(config.logging_handler)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    @property
    def config(self):
        return self._config

    @property
    def log(self):
        return self._log

    @property
    def target(self):
        return "%s:%s" % (self.config.args.host, self.config.args.port)

    def query(self, obj=None):
        if not isinstance(obj, basestring):
            raise TypeError("%s not of type %s" % (obj, basestring))
        result = {
            u'object': unicode(obj)
        }
        return result



import json
import subprocess
from whichcraft import which
from rptk.configuration import Config


class BaseQuerier(object):
    def __init__(self, config=None):
        if not isinstance(config, Config):
            raise TypeError("%s not of type %s" % (config, Config))
        self._config = config

    @property
    def target(self):
        return "%s:%s" % (self._config.args.host, self._config.args.port)

    def query(self, obj=None):
        if not isinstance(obj, str):
            raise TypeError("%s not of type %s" % (obj, str))
        result = {
            'object': obj
        }
        return result


class Bgpq3Querier(BaseQuerier):
    def query(self, obj=None):
        super(Bgpq3Querier, self).query(obj=obj)
        result = dict()
        if not self.path:
            raise RuntimeError("couldn't determine bgpq3 executable path")
        cmds = {
            'ipv4': [self.path, "-h", self.target, "-l", "ipv4", "-4Aj", obj],
            'ipv6': [self.path, "-h", self.target, "-l", "ipv6", "-6Aj", obj]
        }
        for key in cmds:
            result.update(json.loads(subprocess.check_output(cmds[key])))
        return result

    @property
    def path(self):
        try:
            return self._config.args.bgpq3_path
        except AttributeError:
            return which("bgpq3")



import socket
import re
import ipaddress
from pkg_resources import get_distribution
from rptk.modules.query import BaseQuery


class NativeQuery(BaseQuery):
    def __init__(self, config=None):
        super(NativeQuery, self).__init__(config=config)
        self._regexp = re.compile('(?P<state>[ACDEF])(?P<len>\d*)(?P<msg>[\w\s]*)$')
        self._keepalive = True

    def __enter__(self):
        self._connect()
        return self

    def __exit__(self, typ, value, traceback):
        self._disconnect()

    def query(self, obj=None):
        super(NativeQuery, self).query(obj=obj)
        result = dict()
        sets = {u'ipv4': set(), u'ipv6': set()}
        members = self._members(obj=obj)
        for member in members:
            routes = self._routes(obj=member)
            for af in routes:
                sets[af].update(routes[af])
        for af in sets:
            prefixes = sorted(list(sets[af]))
            result[af] = [{u'prefix': p.with_prefixlen, u'exact': True} for p in prefixes]
        return result

    def _connect(self):
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._socket.connect((self.config.args.host, self.config.args.port))
        if self._keepalive:
            self._socket.send('!!\n')
        self._query('!nRPTK-%s' % get_distribution('rptk').version)

    def _disconnect(self):
        self._socket.shutdown(socket.SHUT_RDWR)
        self._socket.close()

    def _query(self, q):
        q += '\n'
        ttl = 0
        sz = len(q)
        while ttl < sz:
            sent = self._socket.send(q[ttl:])
            if not sent:
                raise RuntimeError("socket connection broken")
            ttl = ttl + sent
        self.log.debug("sent %s %d", q.rstrip(), ttl)
        chunks = []
        chunk_size = 4096
        chunk = self._socket.recv(chunk_size)
        response, chunk = chunk.split('\n', 1)
        sz = self._parse_response(response)
        if not sz:
            return
        ttl = len(chunk) or 0
        chunks.append(chunk)
        while ttl <= sz:
            self.log.debug("ttl %d, sz %d", ttl, sz)
            chunk = self._socket.recv(chunk_size)
            if chunk == '':
                raise RuntimeError("socket connection broken")
            chunks.append(chunk)
            ttl += len(chunk)
        self.log.debug("ttl %d, sz %d", ttl, sz)
        suffix = chunks[-1][-(ttl - sz):]
        chunks[-1] = chunks[-1][:-(ttl - sz)]
        self.log.debug("suffix %d '%s'", ttl - sz, suffix)
        return ''.join(chunks)

    def _parse_response(self, response):
        self.log.debug("response %s", response)
        match = self._regexp.match(response)
        if not match:
            raise RuntimeError("invalid response '%s'" % (response,))
        state = match.group('state')
        if state == 'A':
            return int(match.group('len'))
        elif state == 'C':
            return False
        elif state == 'D':
            self.log.warning("skipping key not found")
            return False
        elif state == 'E':
            raise KeyError("multiple copies of key in database")
        elif state == 'F':
            if match.group('msg'):
                msg = match.group('msg').strip()
            else:
                msg = 'unknown error'
            raise RuntimeError(msg)
        raise RuntimeError("invalid response '%s'" % (response,))

    def _members(self, obj=None):
        """ resolve an as-set to its members """
        q = "!i%s,1" % obj
        members = self._query(q)
        if members:
            return members.split()
        else:
            return [obj]

    def _routes(self, obj=None):
        """ get routes for specified object """
        proto = {
            u'ipv4': {'cmd': '!g', 'class': ipaddress.IPv4Network},
            u'ipv6': {'cmd': '!6', 'class': ipaddress.IPv6Network}
        }
        routes = dict()
        for af in proto:
            cmd = proto[af]['cmd']
            cls = proto[af]['class']
            q = "%s%s" % (cmd, obj)
            routes[af] = list()
            resp = self._query(q)
            if resp:
                for each in resp.split():
                    try:
                        routes[af].append(cls(unicode(each)))
                    except ipaddress.AddressValueError, ipaddress.NetmaskValueError:
                        self.log.warning("converting %s to %s failed" % (each, cls))
        return routes



from rptk.modules.format import BaseFormat


class TestFormat(BaseFormat):
    def format(self, result=None, name=None):
        super(TestFormat, self).format(result=result, name=name)
        output = {
            unicode(name): result
        }
        return output



import json
import subprocess
from rptk.modules.query import BaseQuery
from whichcraft import which


class Bgpq3Query(BaseQuery):
    def query(self, obj=None):
        super(Bgpq3Query, self).query(obj=obj)
        result = dict()
        if not self.path:
            raise RuntimeError("couldn't determine bgpq3 executable path")
        cmds = {
            'ipv4': [self.path, "-h", self.target, "-l", "ipv4", "-4Aj", obj],
            'ipv6': [self.path, "-h", self.target, "-l", "ipv6", "-6Aj", obj]
        }
        for key in cmds:
            result.update(json.loads(subprocess.check_output(cmds[key])))
        return result

    @property
    def path(self):
        try:
            return self.config.args.bgpq3_path
        except AttributeError:
            return which("bgpq3")
